#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace HeuristicLab.Core {
  /// <summary>
  /// Interface to represent parameters of operators.
  /// </summary>
  public interface IParameter : IItem {
    /// <summary>
    /// Gets or sets the actual name of the parameter.
    /// </summary>
    string ActualName { get; set; }
    /// <summary>
    /// Gets or sets the formal name of the parameter.
    /// </summary>
    string FormalName { get; }
    /// <summary>
    /// Gets or sets the description of the current instance.
    /// </summary>
    string Description { get; set; }
    /// <summary>
    /// Gets the data type of the current instance.
    /// </summary>
    Type DataType { get; }
    /// <summary>
    /// Gets the type of the parameter (in, out, both).
    /// </summary>
    ParameterType Type { get; }
    /// <summary>
    /// Gets or sets, if the parameter is constant (only valid for input parameters).
    /// </summary>
    bool Constant { get; set; }
    /// <summary>
    /// Gets or sets, if the parameter should be cached by operators.
    /// </summary>
    bool Cached { get; set; }
    /// <summary>
    /// Gets or sets the value of the parameter.
    /// </summary>
    IItem Value { get; set; }

    /// <summary>
    /// Occurs when the actual name of the current instance has been changed.
    /// </summary>
    event EventHandler ActualNameChanged;
    /// <summary>
    /// Occurs when the description of the current instance has been changed.
    /// </summary>
    event EventHandler DescriptionChanged;
    /// <summary>
    /// Occurs when the constant flag of the current instance has been changed.
    /// </summary>
    event EventHandler ConstantChanged;
    /// <summary>
    /// Occurs when the cached flag of the current instance has been changed.
    /// </summary>
    event EventHandler CachedChanged;
    /// <summary>
    /// Occurs when the value of the current instance has been changed.
    /// </summary>
    event EventHandler ValueChanged;

  }

  /// <summary>
  /// Generic interface to represent parameters of operators.
  /// </summary>
  /// <typeparam name="T">The data type of the parameter, which has to be an <see cref="IItem"/>.</typeparam>
  public interface IParamter<T> : IParameter where T : class, IItem {
    /// <summary>
    /// Gets or sets the value of the parameter.
    /// </summary>
    T Value { get; set; }
  }
}
